@@ -1,8 +1,5 @@
-from stuphos.system.game import *
-
 import world
 from world.player import nanny
-from world import peer, mobile_instance
 
 
 def syslog(message):
