@@ -23,7 +23,10 @@ class Programmer:
 		pass
 
 class Script:
-	pass
+	@classmethod
+	def debugInstruction(self, *args, **kwd):
+		pass
+
 
 class Volume:
 	pass
@@ -52,3 +55,6 @@ def findUserByName(*args, **kwd):
 
 def findCurrentUser(*args, **kwd):
 	pass
+
+class nullproc:
+	pass
