@@ -1,65 +1,101 @@
 # Adapter entity types.
-from stuphos.triggers import EntityAdapter
-from stuphos.triggers import TriggerAdapter
-from stuphos.kernel import BypassReturn, vmCurrentTask
+from stuphos.triggers import EntityAdapter, TriggerAdapter
+from stuphos.kernel import BypassReturn, vmCurrentTask, Continuation, accessItemsNative, nullproc
+from stuphos.runtime.architecture import writeprotected
+# from stuphos.etc.tools.misc import accessItems as _accessItems
+
+from sys import exc_info
+
+class EntityNoAccessException(AttributeError):
+    pass
 
 class InlineObject:
     def _getWrappedAttribute(self, name):
-        # Asynchronous message passing of getattr to heartbeat.
         from world import heartbeat as vm
-        task = vmCurrentTask()
+        if vm.isPulseExecuting:
+            return getattr(self._object, name)
+
+        return self._getAttributeForTask(vm, vmCurrentTask(), name)
+
+    def _getAttributeForTask(self, vm, task, name):
+        # Asynchronous message passing of getattr to heartbeat.
+        st = vm.suspendedTasks
 
         def getAttribute():
-            vm.resumeTask(task, value = getattr(self._object, name))
+            # Todo: use vm.threadContext and outer frame completion for dynamic getAttr$?
+            # No: if this is an inline object, a wrapped attribute should always be layer
+            # zero.
+            try: value = getattr(self._object, name)
+            except:
+                st.resumeTask(task, vm, exception = exc_info())
+            else:
+                st.resumeTask(task, vm, value = value)
 
-        vm.suspendTask(task)
+        st.suspendTask(task)
         vm.enqueueHeartbeatTask(getAttribute)
 
         raise BypassReturn
 
+class perspective(EntityAdapter):
+    # Contains perspective.
+    # Todo: rename to yet another class named 'scoped'
+    def __init__(self, object, perspective = None):
+        self._perspective = perspective
+        EntityAdapter.__init__(self, object)
 
-class PeerExtensionPoint(EntityAdapter):
-    # {"valid",            (getter)peer_isvalid},
-    # {"avatar",           (getter)peer_get_avatar},
-    # {"original",         (getter)peer_get_original},
-    # {"authority",        (getter)peer_get_authority},
+    def _rootPerspective(self):
+        if self._perspective is None:
+            return self
 
-    # // Functions as a method generator, too
-    # {"state",            (getter)peer_get_state, (setter)peer_set_state},
+        return self._perspective._rootPerspective()
 
-    # {"input",            (getter)peer_get_next_command, (setter)peer_force_input},
-    # {"large_outbuf",     (getter)peer_get_large_outbuf},
+    def _perspectiveIs(self, types):
+        return isinstance(self._perspective, types)
+    _isPerspective = _perspectiveIs
 
-    # {"input_queue",      (getter)peer_get_input_block_queue},
+    def _perspectiveCheck(self, types, *args, **kwd):
+        if not self._isPerspective(types):
+            exceptionClass = kwd.pop('exceptionClass', AttributeError)
+            raise exceptionClass(*args, **kwd)
 
-    # // Use the output functions to change.
-    # // {"output",           (getter)peer_get_output, (setter)peer_set_output},
 
-    # {"echo",             (getter)NULL, (setter)peer_set_line_echo_state},
-    # {"prompt",           (getter)peer_get_prompt},
+class verbal:
+    # Implements verbs.
 
-    # {"interpreter",      (getter)peer_get_interpreter, (setter)peer_set_interpreter},
-    # #if 0
-    #     {"handler",          (getter)peer_get_interpreter, (setter)peer_set_interpreter},
-    #     {"line_handler",     (getter)peer_get_interpreter, (setter)peer_set_interpreter},
-    #     {"__interpreter__",  (getter)peer_get_interpreter, (setter)peer_set_interpreter},
-    # #endif
+    # @property
+    # def verbs(self):
+    #     return self._object.verbs
+    # @property
+    # def interface(self):
+    #     return self._object.interface
 
-    # {"olcmob", (getter)peer_get_edit_mob},
-    # {"olcobj", (getter)peer_get_edit_obj},
 
-    # #ifdef EXPOSE_DICT
-    #     {"__dict__", (getter)peer_get_dict},
-    # #endif
+    def verbCallObject(self, name):
+        'Do command namespace lookup and return verb action.'
 
-    # {"fileno", (getter)peer_get_fileno},
-    # {"socket", (getter)peer_get_fileno},
+        # debugOn()
+        try:
+            i = self._object._interface
+            a = self._object._verbs.abbrevs[name] # areas/generic/roomVerbs
 
-    # {"network_mode", (getter)peer_get_network_mode, (setter)peer_set_network_mode},
-    # {"networkMode",  (getter)getPeerNetworkMode,    (setter)setPeerNetworkMode   },
+            if isinstance(a, str):
+                a = a.split('.')
 
-    # {"host",     (getter)peer_get_hostname, (setter)peer_set_hostname},
-    # {"hostname", (getter)peer_get_hostname, (setter)peer_set_hostname},
+            # Do effective swapFrameCall
+            task = vmCurrentTask()
+            # task -= task.frame
+
+            # return accessItemsNative(task.frame, i.commands, *a)
+
+            i._invoke(task.frame, dict(), 'lookup$', *['commands'] + a)
+
+        except (AttributeError, KeyError):
+            pass
+
+
+class PeerExtensionPoint(perspective):
+    _readonly_properties = ['state', 'prompt', 'host']
+    __public_members__ = ['input', 'output']
 
     # MSTR(   last_host),
     # MSTR(  olc_string),
@@ -126,49 +162,114 @@ class PeerExtensionPoint(EntityAdapter):
     # // MINT(showstr_vector),
     # MINT(       max_str),
 
-    # {"sendln", (PyCFunction)peer_sendln, METH_VARARGS,
-    #  peer_sendln_doc},
-    # {"textout", (PyCFunction)peer_textout, METH_VARARGS,
-    #  peer_textout_doc},
-    # {"write", (PyCFunction)peer_textout, METH_VARARGS,
-    #  peer_textout_doc},
+    def sendln(self, message):
+        'peer.sendln("text string message without newline")'
 
-    # // Other Output Routines
-    # {"page",         (PyCFunction)peer_page_string, METH_VARARGS, ""},
-    # {"page_string",  (PyCFunction)peer_page_string, METH_VARARGS, ""},
+        # debugOn()
+        return self._object.sendln(message)
 
-    # {"clear_output", (PyCFunction)peer_clear_output, METH_NOARGS, "Reset the output buffers to contain nothing."},
 
-    # // Functions as a getset getter, too
-    # {"next_command", (PyCFunction)peer_get_next_command, METH_NOARGS, ""},
+    @property
+    def avatar(self):
+        return MobileAdapter(self._object.avatar, perspective = self)
+
+    @property
+    def session(self):
+        s = getattr(self._object, 'session_ref', None)
+        if s is not None:
+            s = s()
+            if s is not None:
+                return s.adapter
+
+                # Return a new adapter instance?  No: cleanup on session is used during disconnection.
+                from phsite.network.adapter.sessions import SessionAdapter
+                return SessionAdapter(s)
+
+
+    # interpreter = property(_get_interpreter, _set_interpreter)
+    # messenger = property(_get_messenger, _set_messenger)
+
+    @property
+    def input(self):
+        return self._object.input
+    @input.setter
+    def input(self, input):
+        self._object.input = input
+
+    @property
+    def output(self):
+        return self._object.output
+    @output.setter
+    def output(self, output):
+        self._object.output = output
+
+    def clearOutput(self):
+        return self._object.clear_output()
+    def editString(self, initial):
+        return self._object.editString(initial)
+    def next_command(self):
+        return self._object.next_command()
+    def page_string(self, string):
+        return self._object.page_string(string)
+    page = page_string
+
+    def textout(self, text):
+        return self._object.textout(text)
+    def write(self, *args, **kwd):
+        return self._object.write(*args, **kwd)
+    def forceInput(self, input):
+        return self._object.forceInput(input)
+    def handleCommand(self, comm):
+        return self._object.handleCommand(comm)
 
-    # {"dequeue_input_block", (PyCFunction)peer_dequeue_input_block, METH_NOARGS, ""},
 
     # {"redirect", (PyCFunction)peer_redirect, METH_VARARGS, "A more broad/low-level form of intercepting output."},
 
     # {"members", (PyCFunction)peer_members, METH_NOARGS, ""},
 
-    # {"string_edit", (PyCFunction)peerStringEdit, METH_VARARGS, ""},
-    # {"editString",  (PyCFunction)peerStringEdit, METH_VARARGS, ""},
-
-    # {"withdraw",    (PyCFunction)peerWithdraw,   METH_NOARGS,
-    #  peerWithdrawDoc},
     # {"reload",      (PyCFunction)peerReload,     METH_VARARGS,
     #  peerReloadDoc},
 
-    pass
-
 PeerAdapter = PeerExtensionPoint
 
-class MobileExtensionPoint(InlineObject, EntityAdapter):
-    # {"peer", (getter)mobile_get_peer},
+
+class MobileExtensionPoint(InlineObject, perspective):
+    @property
+    def peer(self):
+        peer = self._object.peer
+        if peer is not None:
+            # Permit object retrieval, but control its scope.
+            return PeerAdapter(peer, perspective = self)
+
+    @property
+    def name(self):
+        return self._object.name
+
+
+    @property
+    def location(self):
+        room = self._object.location
+        if room is not None:
+            return RoomAdapter(room, perspective = self)
+
+    @location.setter
+    def location(self, value):
+        if self._perspective is not None:
+            raise EntityNoAccessException('location')
+        if not isinstance(value, RoomAdapter):
+            raise TypeError(type(value).__name__)
+
+        self._object.location = value._object
+
+    room = location
+
+
     # {"room", (getter)mobile_get_room, (setter)mobile_set_room},
     # {"location", (getter)mobile_get_room, (setter)mobile_set_room},
     # {"next", (getter)mobile_get_next},
 
     # {"zone", (getter)mobile_get_zone},
 
-    # {"name",   (getter)mobile_get_name},
     # {"vnum",   (getter)mobile_get_vnum},
 
     # {"keywords", (getter)mobile_get_keywords, (setter)mobile_set_keywords},
@@ -400,9 +501,6 @@ class MobileExtensionPoint(InlineObject, EntityAdapter):
 
     # MNINT(mob_specials.timer, "timer"),
 
-    # {"perform", (PyCFunction)mobile_perform, METH_VARARGS,
-    #  mobile_perform_doc},
-
     # {"findplayer", (PyCFunction)mobile_findplayer, METH_VARARGS,
     #  mobile_findplayer_doc},
     # {"findchar",   (PyCFunction)mobile_findchar,   METH_VARARGS,
@@ -504,6 +602,137 @@ class MobileExtensionPoint(InlineObject, EntityAdapter):
     # {"consider", (PyCFunction)mobileConsider, METH_VARARGS,
     #  "Return consideration text between combatants.\n"},
 
-    pass
+
+    def sendln(self, message):
+        return self._object.peer.sendln(message)
+
+    def perform(self, command):
+        return self._object.perform(command)
+    __call__ = perform
+
+
+    def interpretCommandInternal(self, command, arguments):
+        return self._object.interpretCommandInternal(command, arguments)
+
+    @property
+    def securityContext(self):
+        from phsite.network.adapter.sessions import SessionAdapter
+        self._perspectiveCheck(SessionAdapter, 'Security context not available!')
+
+        from ph.interpreter.mental.native import _securityContext
+        from stuphos.kernel import Programmer
+
+        # XXX billable?
+        return _securityContext(Programmer(self._object.name))
+
 
 MobileAdapter = MobileExtensionPoint
+
+
+class ZoneResetAccess(writeprotected):
+    '''
+    A handle for providing restricted access to the zone object.
+    It allows for the script to do anything that the zone can do.
+    '''
+
+    __public_members__ = ['initialized']
+
+    @classmethod
+    def _Get(self, zone):
+        try: return zone.accessHandle
+        except AttributeError:
+            return self(zone) # Ephemeral.
+
+            zone.accessHandle = a = self(zone)
+            return a
+
+    def __init__(self, zone):
+        self.__zone = zone
+
+    def __repr__(self):
+        return '{zone #%d %r}' % (self.vnum, self.name)
+
+    @property
+    def initialized(self):
+        return getattr(self.__zone, '_initialized', False)
+    @initialized.setter
+    def initialized(self, value):
+        self.__zone._initialized = bool(value)
+
+    @property
+    def vnum(self):
+        return self.__zone.vnum
+
+    @property
+    def name(self):
+        return self.__zone.name
+
+
+    @property
+    def rooms(self):
+        rooms = (RoomAdapter(r, perspective = self) for r in self.__zone.rooms)
+        return vmCurrentTask().sequence(rooms)
+
+
+    # @property
+    # def mobiles(self):
+    #     return dict((m.vnum, m) for m in map(self._MobilePrototypeAccess, self.__zone.mobiles))
+
+    class _MobilePrototypeAccess(object):
+        def __init__(self, mobile):
+            self.__mobile = mobile
+
+        @property
+        def vnum(self):
+            return self.__mobile.vnum
+
+        @property
+        def special(self):
+            return self.__mobile.special
+
+        @special.setter
+        def special(self, procedure):
+            # Todo: implement access-sensitive (secure) synchronous girl procedure.
+            pass
+
+
+ZoneSystemAccess = (ZoneResetAccess,)
+
+class RoomExtensionPoint(verbal, perspective):
+    _readonly_properties = []
+
+    __public_members__ = ['interface', 'verbs']
+
+    def __repr__(self):
+        # Todo: repr(self.name) if self._perspectiveIs(MobileAdapter) and self._perspective._object.canSeeRoom(self._object) else '<darkness>'
+        return f'<room$adapter {repr(self._object.name)}>'
+
+    @property
+    def vnum(self):
+        self._perspectiveCheck(ZoneSystemAccess, 'Vnum not available!')
+        return self._object.vnum
+
+
+    @property
+    def interface(self):
+        # Todo: for ultimate PeerAdapter perspective, return constrainStructureMemory copy of interface.
+        self._perspectiveCheck(ZoneSystemAccess, 'Interface not available!')
+        return getattr(self._object, '_interface', None)
+
+    @interface.setter
+    def interface(self, value):
+        self._perspectiveCheck(ZoneSystemAccess, 'Interface not available!')
+        self._object._interface = value
+
+    @property
+    def verbs(self):
+        # Todo: for ultimate PeerAdapter perspective, use verbCall API method to access.
+        self._perspectiveCheck(ZoneSystemAccess, 'Verbs not available!')
+        return getattr(self._object, '_verbs', None)
+
+    @verbs.setter
+    def verbs(self, value):
+        self._perspectiveCheck(ZoneSystemAccess, 'Verbs not available!')
+        self._object._verbs = value
+
+RoomAdapter = RoomExtensionPoint
