@@ -471,8 +471,12 @@ def doSelectRichPager(peer, cmd, argstr):
 # the hook install it can be called directly.
 class RichText(Component):
     def onNewConnection(self, cltr, peer):
-        HookPeerEditor(peer)
-        HookPeerPager(peer)
+        try:
+            HookPeerEditor(peer)
+            HookPeerPager(peer)
+
+        except Exception as e:
+            print(f'[rich text:new connection] {e.__class__.__name__}: {e}')
 
 
 @ACMDLN('libedit', Option('--list-directory', '--directory',
