@@ -33,26 +33,19 @@ class World:
     def getZoneConfigFile(self):
         return getConfig('zone-config-file') or self.ZONE_CONFIG_FILE
 
+    from stuphmud.server.adapters import ZoneResetAccess
+
     def onResetStart(self, ctlr):
         self.zoneModules = loadZoneFile(self.getZoneConfigFile())
+
+        # Import zone module associations.
         for module in self.zoneModules:
             try: module.importModule()
             except ImportError:
                 name = module.name and ('[%s]' % module.name) or ''
                 log('Unknown Zone Module%s: %r' % (name, module.package))
 
-        # Do this first, before resetting the world.
-        try:
-            loadResetPrograms()
-            loadTriggers()
-            loadPlanets(self) # the unnamed solar system
 
-        except TypeError as e:
-            # Ths operation did not successfully complete.
-            log('ERROR: %s' % (e,))
-
-
-    def onResetComplete(self, ctlr):
         # Initialize zone module associations.
         for module in self.zoneModules:
             # Configure special procedures first (that they may be overridden by module).
@@ -68,6 +61,18 @@ class World:
                         try: loader(self, module, zone, handle)
                         except: logException(traceback = True)
 
+
+        # Do this first, before resetting the world.
+        try:
+            loadResetPrograms()
+            loadTriggers()
+            loadPlanets(self) # the unnamed solar system
+
+        except TypeError as e:
+            # Ths operation did not successfully complete.
+            log('ERROR: %s' % (e,))
+
+
     # Supplemental.
     def onCreateZone(self, ctlr, zone):
         pass
@@ -78,79 +83,33 @@ class World:
     def onUnloadZone(self, ctlr, zone):
         pass
 
-    class ZoneAccess(object):
-        '''
-        A handle for providing restricted access to the zone object.
-        It allows for the script to do anything that the zone can do.
-        '''
-
-        @classmethod
-        def Get(self, zone):
-            try: return zone.accessHandle
-            except AttributeError:
-                zone.accessHandle = a = self(zone)
-                return a
-
-        def __init__(self, zone):
-            self.__zone = zone
-
-        def __repr__(self):
-            return '{zone #%d %r}' % (self.vnum, self.name)
-
-        @property
-        def vnum(self):
-            return self.__zone.vnum
-
-        @property
-        def name(self):
-            return self.__zone.name
-
-        @property
-        def mobiles(self):
-            return dict((m.vnum, m) for m in map(self.MobilePrototypeAccess, self.__zone.mobiles))
- 
-        class MobilePrototypeAccess(object):
-            def __init__(self, mobile):
-                self.__mobile = mobile
-
-            @property
-            def vnum(self):
-                return self.__mobile.vnum
-
-            @property
-            def special(self):
-                return self.__mobile.special
-
-            @special.setter
-            def special(self, procedure):
-                # Todo: implement access-sensitive (secure) synchronous girl procedure.
-                pass
-
     def onStartZoneReset(self, ctlr, zone):
         from stuphos.kernel import GirlSystemModule
 
         try: program = zone.reset_program
         except AttributeError: pass
         else:
-            import world
-            task = ZoneResetScript()
-            task.environ.update(dict(zone = self.ZoneAccess.Get(zone)))
-                                     # system = GirlSystemModule.Get()
+            if program is not None:
+                script = program.module # Todo: Invocation(program.module)?
 
-            task.tracing = task.auditInstruction if program.tracing else None
+                from world import heartbeat as vm
+                task = ZoneResetScript()
+                task.environ.update(dict(zone = self.ZoneResetAccess._Get(zone)))
+                                         # system = GirlSystemModule.Get()
 
-            script = program.module
-            script.setEnvironment(task.environ)
-            script.name = '#%d:%s' % (zone.vnum, zone.name)
+                task.tracing = task.auditInstruction if program.tracing else None
 
-            if program.programmer is None:
-                task += script
-            else:
-                # import game
-                # game.syslog('zone-reset-programmer: %r' % program.programmer)
-                task.addFrameCall(script, programmer = program.programmer)
+                script.setEnvironment(task.environ)
+                script.name = '#%d:%s' % (zone.vnum, zone.name)
 
-            world.heartbeat += task
+                if program.programmer is None:
+                    task += script
+                else:
+                    # import game
+                    # game.syslog('zone-reset-programmer: %r' % program.programmer)
+                    task.addFrameCall(script, programmer = program.programmer)
+
+                vm += task
 
 @contextmanager
 def database():
@@ -191,16 +150,28 @@ class ResetProgram:
 
 def loadResetPrograms():
     try:
+        # Even though the db is sqlobject, WebProgrammer is a django-specific
+        # component because, like zone reset input or OLC, it is web by design.
+        #
+        # In a strictly non-django runtime, this routine and the WebProgrammer 
+        # object can be replaced by another source-specific impl.
         from stuphos.db.orm import ZoneResetProgram
-        from phsite.network.olc import WebProgrammer
-    except ImportError:
-        pass
+        from phsite.network.embedded.olc.views import WebProgrammer
+
+    except ImportError as e:
+        print(f'[World Reset Programs] {e.__class__.__name__}: {e}')
+    except Exception as e:
+        print(f'[World Reset Programs] {e.__class__.__name__}: {e}')
+
     else:
         def load(zone):
+            # debugOn()
+
             # todo: catch exception where there's no table programming error.
             # Q: Would it be any faster to select and iterate all db entities, and do world.zone BST lookups?
             for program in ZoneResetProgram.selectBy(zone = zone.vnum):
-                zr = zone.reset_program = ResetProgram(zone, program.source)
+                # print(f'[reset.programs.load] {program.programmer}')
+                zr = zone.reset_program = ResetProgram(zone, program.source.replace('\r', ''))
                 if program.programmer in [None, '[None]', '']:
                     zr.programmer = None
                 else:
@@ -212,6 +183,7 @@ def loadResetPrograms():
 
         import world
         with database():
+            # debugOn()
             world.iterent(world.zone, load)
 
 # from mud.tasks.triggers import loadTriggers, saveTrigger
