@@ -680,14 +680,29 @@ class SharedSenario:
                     p.postMessages(['html-panel', message])
 
 
-from stuphos.runtime.architecture.api import writeprotected
-class ProtectedSenario(writeprotected, SharedSenario, Senario):
-    def isElligible(self, player):
-        # todo: type check
-        return Senario.isElligible(self, player._object)
-    def activate(self, player, values = dict()):
-        # todo: type check
-        return Senario.activate(self, player._object, **values)
+# from stuphos.runtime.architecture.api import writeprotected
+# class ProtectedSenario(writeprotected, SharedSenario, Senario):
+#     def isElligible(self, player):
+#         # todo: type check
+#         return Senario.isElligible(self, player._object)
+#     def activate(self, player, values = dict()):
+#         # todo: type check
+#         return Senario.activate(self, player._object, **values)
+
+from stuphos.runtime.architecture import wrappedObject
+class ProtectedSenarioClass(SharedSenario, Senario):
+    pass
+
+ProtectedSenarioClass._definition = wrappedObject._deny \
+    (ProtectedSenarioClass,
+     'Activation',
+     'DEFAULT_BEGIN_MESSAGE',
+     'DEFAULT_END_MESSAGE',
+     'activations')
+
+def ProtectedSenario(**info):
+    return wrappedObject(ProtectedSenarioClass._definition,
+                         ProtectedSenarioClass(**info))
 
 
 # todo: move these into the Senario class.
