@@ -1,5 +1,5 @@
 # Application component library.
-# Copyright 2010-2020 Clint Banis. phaseware project.  All rights reserved.
+# Copyright 2010-2021 runphase.com .  All rights reserved.
 # --
 # The core library made out of components and frameworks for building applications.
 # Applications are rather specific, but most of the system or platform features are
@@ -16,6 +16,6 @@
 from .runtime.core import *
 
 
-try: wmc = io.here.resources('site.wmc').loading.structure
-except: pass
-else: globals().update(dictOf(wmc))
+# try: wmc = io.here.resources('site.wmc').loading.structure
+# except: pass
+# else: globals().update(dictOf(wmc))
