@@ -2,6 +2,7 @@
 from sqlobject import sqlhub, SQLObject, connectionForURI
 from sqlobject import IntCol, StringCol
 from sqlobject import DateTimeCol, FloatCol
+from sqlobject import ForeignKey
 
 
 SmallIntCol = IntCol
@@ -665,6 +666,7 @@ try:
         type = StringCol()
         text = StringCol()
         programmer = StringCol() # node: also used as documument content-type
+        meta = StringCol() # for storing activity superclass, etc.
 
     GirlNode = GirlLibrary
     ProgramLibrary = GirlLibrary
@@ -684,6 +686,7 @@ except ValueError: # Already in registry.
 
 try:
     class GirlLibraryPerm(SQLObject):
+        owner = StringCol()
         principal = StringCol()
         resource = StringCol()
         access = StringCol()
@@ -701,6 +704,15 @@ try:
 except ValueError: # Already in registry.
     pass
 
+try:
+    class GirlCompiledModule(SQLObject):
+        node = ForeignKey('GirlLibrary')
+        signature = StringCol() # checksum
+        payload = StringCol()
+
+except ValueError: # Already in registry.
+    pass
+
 try:
     # Girl System Library replacements.
     class Books(SQLObject): pass
@@ -823,13 +835,17 @@ class VariableDBNativeTool:
 
     def db(self, frame, name, *args, **kwd):
         from stuphos.kernel import interpreter as girl # XXX Make this global!
+
         progr = frame.task.findProgrammer()
+        name = str.__str__(name)
         resource = ['service/database'] + name.split('/')
-        if progr is None or not self._core.principalHasAccess(progr.principal, resource, 'read+write'):
+
+        if progr is None or not self._core.principalHasAccess(progr.principal, resource, 'read+write'): # write
             raise girl.NoAccessException(progr, resource, 'read+write')
 
         tables = dict((t.name, t) for t in args if not t.name.endswith('acct_'))
         return self._vardb._Open(name, **tables)
+
     def table(self, frame, name, *cols):
         cols = [(c.name, c) for c in cols]
         return self._vardb.Table(name, cols)
