@@ -4,9 +4,9 @@ from time import time as getCurrentSystemTime
 
 from datetime import datetime
 
-def date():
+def getCurrentTime():
 	return datetime.fromtimestamp(now())
-
+date = getCurrentTime
 
 # todo: better naming
 class Elapsed:
