@@ -3,19 +3,34 @@
 from . import structural as _document
 getContextVariable = _document.Context.__getitem__
 
-from stuphos.structure import Factory, SystemFactory
+from stuphos.kernel import Machine as VM
 
-def getContextEnvironment(name):
-	return getContextVariable('loader').environ[name]
+def getContextEnvironment(name, **kwd):
+	loader = getContextVariable('loader')
+
+	try: default = kwd['default']
+	except KeyError:
+		return loader.environ[name]
+	else:
+		return loader.environ.get(name, default)
+
+from stuphos.structure import Factory, SystemFactory
 
 def load(source, classes, defaultName, **kwd):
 	source = source.replace('\r', '') # scrub from any source.
 	# source = 'Westmetal Configuration::\n\n' + source
 	source = 'WMC []\n\n' + source
 
-	return _document.loadStructuredMessageFromClasses \
-			(source, classes, raw = True,
-			 default = defaultName, **kwd)
+	# debugOn()
+	try: return _document.loadStructuredMessageFromClasses \
+				 (source, classes, raw = True,
+				  default = defaultName, **kwd)
+
+	except VM.Yield as e:
+		# Don't propogate continuations because the document load will be
+		# replaced by the yielding factory load, which might be confusing.
+		raise RuntimeError('Incompatible continuation stack!') from e
+
 
 def getFactories():
 	yield 'stuph', Factory
