@@ -1,6 +1,6 @@
 # From WRLC
 # Typed Yaml -- Revelations 7-12
-#  Copyright 2011-2014 Clint Banis.  All rights reserved.
+# Copyright 2021 runphase.com .  All rights reserved
 #  
 # Notes:
 #   OrderedDict? (omap)
@@ -369,7 +369,7 @@ class StructuredEncoding:
                                      ', '.join(list(self.keys())))
 
             def __str__(self):
-                return json.dumps(self, indent = 1)
+                return json.dumps(self, indent = 1, default = lambda o: None)
 
         # Factory.
         def buildItem(self, loader, subtype, name, value, **kwd):
@@ -384,7 +384,7 @@ class StructuredEncoding:
 
                 return self.Item.FromStructure(value)
 
-                # XXX Get rid of the runtime-error checking
+                # XXX :skip: Get rid of the runtime-error checking
                 ##    try: return self.Item.FromStructure(value)
                 ##    except RuntimeError:
                 ##        builtin(_ = value)
@@ -396,7 +396,7 @@ class StructuredEncoding:
                 ##                           (loader, None, n, v, **kwd))
                 ##                          for (n, v) in value.iteritems()))
 
-                # XXX I don't see why this should _ever_ happen from here.
+                # XXX :skip: I don't see why this should _ever_ happen from here.
                 # This recursion should be happening in the Building class.
                 #
                 # I think this is the strategy for yielding an Item, but,
@@ -422,7 +422,7 @@ class StructuredEncoding:
             ##        return [self.buildItem(loader, None, None, v, **kwd)
             ##                for v in value]
             ##
-            ##        # XXX I don't see why this should _ever_ happen from here.
+            ##        # XXX :skip: I don't see why this should _ever_ happen from here.
             ##        # This recursion should be happening in the Building class.
             ##        return [loader.classes.buildTypedItemMap(loader, v)
             ##                for v in value]
@@ -709,9 +709,14 @@ class StructuredEncoding:
         def buildToplevelItem(self, loader, object):
             # Todo: redevelop my notion of what the toplevel is.
             # Also, see Submapping.
+            # debugOn()
+
             with Context(loader = loader, root = object):
                 object = self.buildTypedItemMap(loader, object)
 
+            if isinstance(object, (list, tuple)):
+                return object # Do no mapping of key-values.
+
             return StructuredEncoding.ItemClass.Item(object)
 
 
@@ -1024,6 +1029,9 @@ class Submapping(StructuredEncoding.ItemClass):
 
     def getTypeImpl(self, name):
         self.resolveAliases()
+        if name.startswith('_'):
+            raise NameError(f'Type implementation name must not start with underscore: {name}')
+
         return getattr(self, name, None) # i guess
 
 
