@@ -1,5 +1,5 @@
 from . import CommandMessage, AsCommandClass
-import ..deploy # xxx dependence
+from .. import deploy # xxx dependence
 deploymentApi = runtime.MUD.Deployment.API
 
 @AsCommandClass('application/x-stuph-deployment')
