@@ -17,6 +17,9 @@ Implementation:
         
 """
 
+from stuphos.etc.tools.strings import compactify
+from stuphos.runtime.architecture.api import NoAccessException
+
 import re, errno, pickle, traceback,pprint
 from os import popen
 
@@ -283,7 +286,7 @@ class InteractiveMatch(PatternMatch):
 
                 print('[%s]' % type.capitalize(), file=out)
 
-            # XXX careful:
+            # XXX :skip: careful:
             pattern = str(pattern).rstrip()
 
             print('%d: %s'  % (nr, pattern), file=out)
@@ -593,7 +596,7 @@ def doScanSyslog(peer, cmd, argstr):
                     for line in stream:
                         linenr += 1
 
-                        # XXX Careful:
+                        # XXX :skip: Careful:
                         line = line.rstrip()
                         line = line[28:]
 
@@ -655,6 +658,7 @@ else: ACMD('syslog*')(doScanSyslog)
 
 
 from stuphos.runtime.facilities import Facility
+from stuphos.db import dbCore
 from stuphos.kernel import Native
 from stuphos.etc.tools.timing import date as now
 from stuphos.etc.tools.logs import tracebackString, exceptionHeader
@@ -679,11 +683,13 @@ class Journal(Facility):
         self.startStream()
 
     def processStream(self, write, get):
+        # print(f'[vsz] starting stream: {psOpGameVsz()}')
         self.running = True
         try:
-            job = None
             while True:
+                job = None
                 try:
+                    # print(f'[vsz] getting journal job: {psOpGameVsz()}')
                     job = get()
                     write(job)
 
@@ -738,7 +744,7 @@ class Journal(Facility):
         if isinstance(msg, StopIteration) or msg is StopIteration:
             raise msg
 
-        # return # XXX absorb all until we can migrate Log
+        # return # XXX :skip: absorb all until we can migrate Log
 
         if isinstance(msg, dict):
             source = msg['source']
@@ -751,6 +757,8 @@ class Journal(Facility):
         from stuphos.db.orm import Log
         from stuphos.db import dbCore
 
+        # print(f'[vsz] writing log: {psOpGameVsz()}')
+
         with dbCore.hubThread(self.dbNamespace):
             try: Log(source = source, type = type,
                      content = content,
@@ -763,14 +771,38 @@ class Journal(Facility):
             except ImportError as e:
                 # Yep.
                 import re
-                pattern = re.compile(r'^Failed to import ([_a-zA-Z0-9]+) because the import lockis held by another thread.$').match
+                pattern = re.compile(r'^Failed to import ([_a-zA-Z0-9]+) because the import lock is held by another thread.$').match
                 if pattern(str(e)) is not None:
                     # Because this is intermittant, we debug it...
                     print(e)
-                    import pdb; pdb.set_trace()
+                    debugOn()
                 else:
                     raise e
 
+        # print(f'[vsz] wrote log: {psOpGameVsz()}')
+
+    def copyToDB(self, fromNS, toNS):
+        from stuphos.db.orm import Log
+        from stuphos.db import dbCore
+
+        with dbCore.hubThread(fromNS):
+            # Copy to memory.
+            data = [(e.source, e.type, e.content, e.timestamp)
+                    for e in Log.select()]
+
+        with dbCore.hubThread(toNS):
+            try: Log.createTable()
+            except Exception as e:
+                print(f'{toNS}: {e}')
+
+            for (source, type, content, timestamp) in data:
+                # Write to database.
+                Log(source = source, type = type,
+                    content = content,
+                    timestamp = timestamp).sync()
+
+        return len(data)
+
 
     def __add__(self, msg):
         self.queue.put(msg)
@@ -781,8 +813,13 @@ class Journal(Facility):
         traceback = '\n'.join(task.formatTraceback(traceback))
         if exception is not None:
             (etype, value, tb) = exception
-            traceback += '\n%s:\n%s\n' % (exceptionHeader(etype, value),
-                                          tracebackString(tb))
+            try: header = exceptionHeader(etype, value)
+            except NoAccessException:
+                try: header = f'{etype.__name__}: {repr(value)}'
+                except NoAccessException:
+                    header = f'{etype.__name__}: id #{id(value)}'
+
+            traceback += '\n%s:\n%s\n' % (header, tracebackString(tb))
 
         self += (task.name, 'traceback', traceback,
                  datetime.fromtimestamp(now()))
@@ -823,6 +860,10 @@ class Journal(Facility):
             return iter(Log.selectBy(**criteria).orderBy('timestamp'))
     __iter__ = iterate
 
+    def deleteAllType(self, type):
+        with dbCore.hubThread(self.dbNamespace):
+            for e in self.iterate(type = type):
+                e.delete(e.id)
 
     def waitLogs(self, task, waiter, once = False):
         # todo: if already available, call waiter.
@@ -835,41 +876,236 @@ class Journal(Facility):
             syslog(line)
 
     @classmethod
-    def formatTraceback(self, traceback):
-        for (f, pos) in traceback:
+    def iterateTracebackData(self, traceback):
+        # Compatibility with raise$()d tracebacks that use $stack.
+        if traceback is not None:
+            try: from ph.emulation.machine.kernel import TracebackFrame, Traceback
+            except ImportError:
+                Traceback = TracebackFrame = type(None)
+
+            if isinstance(traceback, Traceback):
+                # Hide the entire traceback.
+                traceback = traceback._iterate()
+
+            for frameTb in traceback:
+                if frameTb is None:
+                    continue
+
+                if isinstance(frameTb, TracebackFrame):
+                    yield (frameTb._frame, frameTb._pos)
+                else:
+                    yield frameTb
+
+
+    @classmethod
+    def formatTraceback(self, traceback, **kwd):
+        def formatSource(source):
+            return compactify(source, 80).replace('\n', ' ')
+
+        # Build security routines with parameter.
+        try: checkAccess = kwd['checkAccess']
+        except KeyError:
+            def hasAccess(source):
+                return True
+        else:
+            agentCore = runtime[runtime.Agent.System]
+            if not agentCore:
+                def hasAccess(source):
+                    return True
+            else:
+                def hasAccess(source):
+                    if isinstance(source, str):
+                        return True
+
+                    # Otherwise, it's a CodeSource.
+                    return agentCore.principalHasAccess \
+                        (checkAccess, list(source), 'read')
+
+
+        for (f, pos) in self.iterateTracebackData(traceback):
             p = f.procedure
 
+            if getattr(p, '_omitTraceback', False):
+                continue
+
+            # print(pos, p)
+
             # Todo: get source code line number from position-to-linenr table,
             # or, yield the Program disassembly of the instruction to string.
             # Note: treating position as if it's the next pc to execute.
 
             if isinstance(p, Native): # Or SubNative
-                yield '  %s:' % getattr(p._native, '__name__', None)
+                # XXX
+                try: name = p._native.__name__
+                except: pass # Could be a NoAccessException if the native is an Instance.
+                else: yield '  %s:' % (name,)
+
                 continue
 
+            try: name = p.name
+            except AttributeError:
+                name = ''
+            else:
+                name = f'{name or ""}{name and ":" or ""}'
+
+
             # debugOn() # :traceback:
-            source = p.lineSourceInfo(pos-1)
+            try: source = p.lineSourceInfo(pos-1)
+            except AttributeError:
+                source = None
+
             if source is not None:
                 (lnr, source, code) = source
-                yield '[%s:%d]' % (source.rstrip(), lnr)
-                if code:
-                    yield '  %s' % code
+                if hasAccess(source):
+                    yield f'[{formatSource(source)}:{lnr} ({name}{pos})]'
+
+                    if code:
+                        yield '  %s' % code.lstrip() # Assume no one's pumping ws at end.
+
+                else:
+                    yield f'[{formatSource(source)} ({name})]'
 
             else:
+                try: source = p.getSourceMap().source
+                except AttributeError: source = None
+
+                if source:
+                    yield f'[{formatSource(source)} ({name}{pos})]'
+
                 try: (start, g) = p.lineGroup(pos)
-                except TypeError:
+                except (TypeError, AttributeError):
                     # No line source info found -- render one instruction string.
-                    s = p.instructionsString(pos-1, pos, suppress_name = True)
+                    try: s = p.instructionsString(pos-1, pos, suppress_name = True)
+                    except AttributeError:
+                        s = '<unknown procedure>'
                 else:
-                    s = p.instructionsString(start, start + len(g),
-                                             suppress_name = True,
-                                             highlight = pos)
+                    span = 1 # len(g) # XXX :skip: 20 instructions per line??
+                    if pos >= start and pos <= (start + span):
+                        s = p.instructionsString(start, start + span,
+                                                 suppress_name = True,
+                                                 highlight = pos)
+                    else:
+                        # Why lineGroup isn't returning pos-containing span?
+                        s = p.instructionsString(pos-1, pos, suppress_name = True)
 
                 n = getattr(p, 'name', None)
                 if n:
-                    yield '  %s:' % n
+                    yield f'  {n}'
 
-                yield '    %s' % (s.strip(),)
+                yield indent(s.strip())
+
+
+    # XXX Duplicate code:
+    # Todo: rename to formatTracebackContext
+    @classmethod
+    def formatTracebackHtml(self, traceback, **kwd):
+        def formatSource(source):
+            return compactify(source, 80).replace('\n', ' ')
+
+        # Build security routines with parameter.
+        try: checkAccess = kwd['checkAccess']
+        except KeyError:
+            def hasAccess(source):
+                return True
+        else:
+            agentCore = runtime[runtime.Agent.System]
+            if not agentCore:
+                def hasAccess(source):
+                    return True
+            else:
+                def hasAccess(source):
+                    if isinstance(source, str):
+                        return True
+
+                    # Otherwise, it's a CodeSource.
+                    return agentCore.principalHasAccess \
+                        (checkAccess, list(source), 'read')
+
+
+        for (f, pos) in self.iterateTracebackData(traceback):
+            p = f.procedure
+
+            if getattr(p, '_omitTraceback', False):
+                continue
+
+            # print(pos, p)
+
+            # Todo: get source code line number from position-to-linenr table,
+            # or, yield the Program disassembly of the instruction to string.
+            # Note: treating position as if it's the next pc to execute.
+
+            if isinstance(p, Native): # Or SubNative
+                # XXX
+                try: name = p._native.__name__
+                except: pass # Could be a NoAccessException if the native is an Instance.
+                else: yield '  %s:' % (name,)
+
+                continue
+
+            try: name = p.name
+            except AttributeError:
+                name = ''
+            else:
+                name = f'{name or ""}{name and ":" or ""}'
+
+
+            # debugOn() # :traceback:
+            try: source = p.lineSourceInfo(pos-1)
+            except AttributeError:
+                source = None
+
+            if source is not None:
+                (lnr, source, code) = source
+                if hasAccess(source):
+                    r = dict(sourceFormatted = formatSource(source),
+                             source = source, lineNr = lnr, name = name,
+                             position = pos)
+
+                    if code:
+                        r['code'] = code
+
+                    yield r
+
+                else:
+                    yield dict(sourceFormatted = formatSource(source),
+                               source = source, name = name)
+
+            else:
+                try: source = p.getSourceMap().source
+                except AttributeError: source = None
+
+                r = dict()
+
+                if source:
+                    r['sourceFormatted'] = formatSource(source)
+                    r['source'] = source
+                    r['name'] = name
+                    r['position'] = name
+
+                try: (start, g) = p.lineGroup(pos)
+                except (TypeError, AttributeError):
+                    # No line source info found -- render one instruction string.
+                    try: s = p.instructionsString(pos-1, pos, suppress_name = True)
+                    except AttributeError:
+                        s = '<unknown procedure>'
+                else:
+                    span = 1 # len(g) # XXX :skip: 20 instructions per line??
+                    if pos >= start and pos <= (start + span):
+                        s = p.instructionsString(start, start + span,
+                                                 suppress_name = True,
+                                                 highlight = pos)
+                    else:
+                        # Why lineGroup isn't returning pos-containing span?
+                        s = p.instructionsString(pos-1, pos, suppress_name = True)
+
+                n = getattr(p, 'name', None)
+                if n:
+                    r['codeName'] = n
+
+                r['code'] = s
+                yield r
+
+    formatTracebackContext = formatTracebackHtml
 
 
 from threading import Thread
