@@ -1,6 +1,8 @@
 # Core Content Design and Delivery Client
+# Todo: move into stuphos.management
 # XMLRPC Python API -- requires WRLC
 from xmlrpc.client import ServerProxy, MultiCall
+import ssl
 
 
 HOSTNAME = 'localhost'
@@ -76,9 +78,9 @@ class folderUpload(namespace):
         pass
 
 
-class filesystem:
+class phClientFS:
     '''
-    client.upload(filesystem(args).structure,
+    client.upload(phClientFS(args).structure,
                   path = 'mount/point')
 
     '''
@@ -180,9 +182,25 @@ class apiClient:
 
     def __init__(self, username, password, host = HOST, protocol = 'https', runtime = None, **kwd):
         # Note: xmlrpclib._Method.__call__ does not accept keyword arguments.
-        self.server = ServerProxy('%s://%s:%s@%s/rpc' % (protocol, username, password, host))
+        if username is None and password is None:
+            endpoint = '%s://%s/rpc' % (protocol, host)
+        else:
+            endpoint = '%s://%s:%s@%s/rpc' % (protocol, username, password, host)
+
+        self.host = host
+        self.protocol = protocol
+
+        self.endpoint = endpoint
+
         self.runtime = runtime if runtime is not None else globals()['runtime'](**kwd)
 
+    @property
+    def server(self):
+        if self.protocol == 'https':
+            return ServerProxy(self.endpoint, context = ssl.SSLContext())
+
+        return ServerProxy(self.endpoint)
+
     def run(self, *args, **kwd):
         return self.runtime.run(self, *args, **kwd)
 
@@ -558,6 +576,56 @@ class apiClient:
         upload = packageUpload
 
 
+    def login(self, playerName, password, *args, **kwd):
+        return self.session(self, playerName, password, *args, **kwd)
+    def resume(self, sessionId):
+        return self.session(self, None, None, sessionId = sessionId)
+
+    class session:
+        terminateOnGC = False
+
+        def __init__(self, client, playerName, password, sessionId = None, passwordSend = True):
+            self.client = client
+
+            if sessionId is None:
+                # XXX Todo: session.open method not supported on circle.
+                self.sessionId = self.client.server.session.open(playerName)
+                self.terminateOnGC = True
+
+                if passwordSend:
+                    self.send(password)
+            else:
+                self.sessionId = sessionId
+
+        def __repr__(self):
+            return '<%s %r>' % (self.__class__.__name__, self.sessionId)
+
+        def __del__(self):
+            if self.terminateOnGC:
+                # XXX Todo: transition to cleanup object
+                pass # self.client.server.session.terminate(self.sessionId)
+
+        def send(self, message):
+            return self.client.server.session.send(self.sessionId, message)
+        def command(self, name, *args):
+            # print(f'[phClient.session.command] {name}')
+            return self.client.server.session.command(self.sessionId, name, *args)
+        def messages(self, n = -1, timeout = None):
+            return self.client.server.session.messages(self.sessionId, n, timeout)
+
+        def loginShell(self, password):
+            return self.client.server.session.shell.login(self.sessionId, password)
+
+        # Web Adapter Commands
+        def interpretAgent(self, script):
+            return self.command('do-interpret-agent', script)
+        def interpretScript(self, script):
+            return self.command('do-interpret-script', script)
+
+        def interpretAsyncScript(self, script, nr):
+            return self.command('do-interpret-script-async', script, nr)
+
+
 @actionable
 def stringCall(name, *arguments):
     # Unifies all string states into one string.
@@ -697,8 +765,111 @@ class runtime:
         return function(*args, **kwd)
 
 
-# serverbook = login(501, 1234).node('/house/legal')
-# The Matrix is a joke by the way.  We are currently unable to
-# convincibly dream of such a reality greater than ourselves.
-# However, if we are infinite, then we possess no such limitation.
+class nativeSession:
+    '''
+    itham = 'network/game/session'('runphase.com:2172', 'itham', 'ithams-password')
+    task$(_processSessionMessages, session, itham)
 
+    def _processSessionMessages(session, player):
+        while true:
+            _processMessages(session, player.messages)
+
+    def _processMessages(session, messages):
+        for msg in messages:
+            act(_processMessage, [session] + msg)
+
+    def _processMessage(session, type):
+        msg = arguments().slice(2)
+
+        if type == 'output':
+            session.sendln(msg[0])
+
+    _processMessages(session, itham + 'return $.network.stats.users().display()')
+    itham('look board')
+
+    '''
+
+    def __init__(self, frame, host, playerName, password, passwordSend = True):
+        passwordSend = bool(passwordSend)
+
+        task = frame.task
+        task.checkAccess(['system:network', 'https', host, 'rpc'], 'write')
+        client = apiClient(None, None, host = host) # , protocol = 'http')
+
+        task -= frame
+
+        # print(f'[phClient.session] {client} {task} {self._pool}')
+        # debugOn()
+
+
+        @self._pool.task(task)
+        def connect():
+            from xmlrpc.client import Fault
+
+            print(f'[phClient.session.connect] START: {client} {playerName}')
+
+            try: self._session = client.login(playerName, password, passwordSend = passwordSend)
+            except Fault as e:
+                # XXX Todo: hide system traceback. :skip:
+                raise e
+
+            print(f'[phClient.session.connect] DONE: {self}')
+            return self
+
+    def _action(self, function, *args, **kwd):
+        from stuphos.kernel import vmCurrentTask
+        task = vmCurrentTask()
+        task -= task.frame
+
+        @self._pool.task(task)
+        def action():
+            # from xmlrpc.client import Fault
+
+            try: return function(*args, **kwd)
+            # except Fault as e:
+            except Exception as e:
+                # XXX Todo: hide system traceback. :skip:
+                raise e
+
+            # try: result = function(*args, **kwd)
+            # except Exception as e:
+            #     debugOn()
+            #     raise e
+
+            # debugOn()
+            # return result
+
+    def send(self, message):
+        return self._action(self._session.send, message)
+    def command(self, name, *args):
+        return self._action(self._session.command, name, *args)
+
+    __call__ = send
+
+    # @property
+    def messages(self):
+        return self._action(self._session.messages, timeout = 30)
+
+    def loginShell(self, password):
+        return self._action(self._session.loginShell, password)
+
+    def interpretAgent(self, script):
+        return self._action(self._session.interpretAgent, script)
+    def interpretScript(self, script):
+        return self._action(self._session.interpretScript, script)
+
+    def interpretAsyncScript(self, script, nr):
+        return self._action(self._session.interpretAsyncScript, script, nr)
+
+
+    def __add__(self, script):
+        return self.interpretScript(script)
+    __lshift__ = __add__
+
+
+    @property
+    def _lookbackSession(self):
+        # XXX port
+        if self._session.client.host in ['localhost', '127.0.0.1']:
+            mgr = runtime[runtime.Web.Adapter.SessionManager]
+            return mgr.getSessionByIdEx(None, self._session.sessionId)
