@@ -1,6 +1,9 @@
 # Software Architecture
 from contextlib import contextmanager
-from types import ClassType as newClassObject, ClassType
+# from types import ClassType as newClassObject, ClassType
+newClassObject = type
+TypeType = type
+ClassType = type
 
 def classObject(name, *bases, **values):
 	return newClassObject(name, bases, values)
