@@ -10,6 +10,12 @@ LVL_DEITY = 108
 LVL_IMMORT = 107
 LVL_AVATAR = 106
 
+# debugOn()
+
+# For CircleMUD adapters.
+try: from world import LVL_IMMORT, LVL_IMPL # , LVL_GOD, LVL_GRGOD
+except ImportError: pass
+
 # Aliases.
 LVL_SUPREME = LVL_IMPL
 LVL_FREEZE = LVL_OVERSEER
