@@ -10,6 +10,8 @@ from stuphmud.server.player.interfaces.dictcmd import CmdDict
 from stuphos.kernel import Girl, Script, GirlSystemModule
 from stuphos.system.api import mudlog
 from stuphos.runtime.architecture.api import writeprotected, wrapper
+from stuphos.etc.tools import isYesValue
+from stuphos.etc.tools.logs import tracebackString
 
 #girlSystemModule = runtime.Girl.System.Module
 
@@ -51,10 +53,23 @@ class TriggerScript(Script):
             self.tracing = self.auditInstruction # traceToLog
 
     def handleError(self, task, frame, exc, traceback):
+        # name = "?" if frame is None else getattr(frame.procedure, "name", "?")
+        # logOperation(f'[trigger$error] {name}:' + \
+        #              f' {exc[0].__name__}: {exc[1]}')
+
+
         if not self.handleOperatorError(task, frame, exc, traceback):
             (exc_class, exc_value, exc_tb) = exc
-            mudlog('(%s: %s) %s: %s' % (task.name, getattr(frame.procedure, 'name', '?'),
-                                        exc_class.__name__, exc_value))
+    
+            # XXX :skip: mudlog doesn't log to file (console stderr)
+            def mudlog(s):
+                logOperation('[trigger$error] ' + s)
+
+            mudlog('(%s: %s) %s: %s\n%s' % \
+                (task.name, '?' if frame is None else \
+                 getattr(frame.procedure, 'name', '?'),
+                 exc_class.__name__, exc_value,
+                 tracebackString(exc_tb, indent = 4)))
 
         raise self.Done
 
@@ -81,8 +96,13 @@ class TriggerScript(Script):
     def invokeTrigger(self, program, context, tracing = False,
                       programmer = None, name = None):
 
-        from stuphos.kernel import startTask # XXX This is somehow named wrongly in the interior import.  Where is global startTask??
+        # from stuphos.kernel import startTask # XXX :skip: This is somehow named wrongly in the interior import.  Where is global startTask??
+        from stuphos.kernel import executeGirl
         # context.setdefault('system', GirlSystemModule.Get())
+
+        program = program.replace('\r', '')
+        program = Girl(Girl.Module, program)
+
         if programmer is None:
             user = None
         else:
@@ -92,11 +112,14 @@ class TriggerScript(Script):
         task = self.Load(tracing = tracing, environ = context, user = user) # , programmer = programmer)
         task.name = name
         task.operator = None if programmer is None else programmer.principal
-        program = Girl(Girl.Module, program)
         program.setEnvironment(task.environ)
         program.name = name
         # programmer.invoke(task, program)
-        startTask(task, program, programmer = programmer)
+        # startTask(task, program, programmer = programmer)
+
+        task.addFrameCall(program, programmer = programmer)
+        executeGirl(task)
+        # return task
 
     @classmethod
     def invokeVerbCommand(self, player, target, object):
@@ -177,12 +200,16 @@ def prototypeTriggering(ch, callback):
                 return True
 
     # Try triggers registered for the prototype.
-    try: triggers = ch.prototype.triggers
+    # Note: check identity of triggers object because
+    # entity_instance (mobile_instance) overrides
+    # __getattr__.
+    try: ptriggers = ch.prototype.triggers
     except AttributeError: pass
     else:
-        for (i, trig) in enumerate(triggers):
-            if callback(ch, trig, i):
-                return True
+        if ptriggers is not triggers:
+            for (i, trig) in enumerate(triggers):
+                if callback(ch, trig, i):
+                    return True
 
 def getTriggerName(object, nr, trig):
     return '%s:%d:%s' % (object, nr, getMobileTriggerType(trig).typeCode)
@@ -404,6 +431,8 @@ class CreateMobileTrigger(MobileTrigger):
         return True
 
     def invoke(self, mobile, source, cause, name):
+        # print(f'[create.mobile$trigger] {self.programmer}')
+
         return MobileScript.invokeTrigger \
                (self.program,
                 dict(mobile = mobile,
@@ -417,9 +446,13 @@ class CreateMobileTrigger(MobileTrigger):
     def Event(self, mobile, source, cause):
         def action(object, trig, nr):
             if trig.mobileCreated():
+                # logOperation(f'[create.mobile$trigger] #{nr}: {mobile} -- {trig.program}')
+                # debugOn()
+
                 trig.invoke(mobile, source, cause,
                             getTriggerName(object, nr, trig))
-                return True
+
+                return False # Allow pipelining of triggers.
 
         prototypeTriggering(mobile, action)
 
@@ -673,9 +706,13 @@ def verbCommand(player, command, argstr):
         #     if verb is not None:
         #         program = "call(%r, %r, %r)" % (verb.method, command, argstr)
 
+
         # A verb is a command-based action in the interactive world.
-        from stuphmud.server.player.interfaces.code.girl import PlayerScript
-        return PlayerScript.evaluateVerb(player, ch, command, argstr)
+        # Todo: how to integrate with evaluateCommand?
+        if isYesValue(configuration.Interpreter.evaluate_verbs):
+            from stuphmud.server.player.interfaces.code.girl import PlayerScript
+            return PlayerScript.evaluateVerb(player, ch, command, argstr)
+
 
         # try: spatial = ch.spatial
         # except AttributeError: pass
