@@ -1,4 +1,4 @@
-# StuphMUD Compartmentalized Django WSGI Development/Production Server
+# StuphOS Compartmentalized Django WSGI Development/Production Server
 #   for Django-1.2.1
 
 # This contains a django framework and a bit of an application framework
@@ -19,16 +19,25 @@ from django.core.servers.basehttp import run, WSGIRequestHandler, WSGIServer, Se
 from django.core.handlers.wsgi import WSGIHandler
 from django.conf import settings
 from django.utils import translation
+from django.core.servers.basehttp import WSGIRequestHandler
 
 from stuphos.runtime.facilities import Facility
 from stuphos.runtime.registry import getObject
 from stuphos.runtime import newComponent
+from stuphos.runtime.architecture.api import writeprotected
 
-from io import StringIO
+# Done dynamically:
+# from stuphos.structure import RequestAdapter
+
+from io import StringIO, BytesIO
 from _thread import start_new_thread as nth
+from threading import Thread
 from select import error as select_error
 from errno import EINTR
 from contextlib import contextmanager
+from socketserver import BaseServer
+
+from http.client import HTTPResponse as baseHTTPResponse
 
 import os
 import sys
@@ -112,6 +121,9 @@ class ServerControl(ServerCompartment):
             while self.is_running():
                 #import pdb; pdb.set_trace()
 
+                # print('handling django request...')
+                # debugOn()
+
                 try: self.handle_request()
                 except select_error as e:
                     if e.args[0] is EINTR:
@@ -127,7 +139,8 @@ class ServerControl(ServerCompartment):
                     self.server_stop()
 
         self.set_running(True)
-        nth(serve_loop, ())
+        Thread(target = serve_loop, daemon = True).start()
+        # nth(serve_loop, ())
 
     def server_stop(self):
         self.set_running(False)
@@ -157,28 +170,28 @@ class ServerControl(ServerCompartment):
 
             logException(traceback = True, header = False)
 
-        method = getConfig('process-error') # XXX currently disabled because invocation not called right, 'core/system/httpError')
-        if method:
-            from stuphos.kernel import invokeLibraryMethod as i
-            task = i(method, etype, value, tb,
-                     request, client_address)
+        # method = getConfig('process-error') # XXX :skip: currently disabled because invocation not called right, 'core/system/httpError')
+        # if method:
+        #     from stuphos.kernel import invokeLibraryMethod as i
+        #     task = i(method, etype, value, tb,
+        #              request, client_address)
 
-            @nth
-            def async():
-                # Why does this need to be off-thread again??
-                # I ask because handle_error should already be parallel to VM thread.
+        #     @nth
+        #     def asynchronous():
+        #         # Why does this need to be off-thread again??
+        #         # I ask because handle_error should already be parallel to VM thread.
 
-                # XXX This is also wrong because the task variable is actually bound
-                #   to a result object from a task that has run.
+        #         # XXX :skip: This is also wrong because the task variable is actually bound
+        #         #   to a result object from a task that has run.
 
-                task = task()
+        #         task = task()
 
-                @task.onComplete
-                def completion(task, exception = None):
-                    log()
+        #         @task.onComplete
+        #         def completion(task, exception = None, **kwd):
+        #             log()
 
-        else:
-            log()
+        # else:
+        #     log()
 
 
 class Instrument:
@@ -199,12 +212,30 @@ class EmbeddedRequestHandler(WSGIRequestHandler):
     class ServerHandler(ServerHandler):
         def __init__(self, stdin, stdout, stderr, environ, software, **kwd):
                      # multithread = True, multiprocess = False):
-            ServerHandler.__init__(self, stdin, stdout, stderr, environ, software, **kwd)
+            ServerHandler.__init__(self, stdin, stdout, stderr, environ, **kwd)
             self.server_software = software
 
     def handle(self):
         """Copy of WSGIRequestHandler, but with different ServerHandler"""
 
+        # [HTTP 192.168.43.1:52786] ConnectionResetError: [Errno 104] Connection reset by peer [<ssl.SSLSocket fd=15, family=AddressFamily.AF_INET, type=SocketKind.SOCK_STREAM, proto=0, laddr=('192.168.43.192', 2180)>]
+        #   [socketserver.py:654] process_request_thread
+        #     self.finish_request(request, client_address)
+        #   [socketserver.py:364] finish_request
+        #     self.RequestHandlerClass(request, client_address, self)
+        #   [socketserver.py:724] __init__
+        #     self.handle()
+        #   [stuphos/webserver/__init__.py:211] handle
+        #     self.raw_requestline = self.rfile.readline(65537)
+        #   [socket.py:586] readinto
+        #     return self._sock.recv_into(b)
+        #   [ssl.py:971] recv_into
+        #     return self.read(nbytes, buffer)
+        #   [ssl.py:833] read
+        #     return self._sslobj.read(len, buffer)
+        #   [ssl.py:590] read
+        #     v = self._sslobj.read(len, buffer)
+
         self.raw_requestline = self.rfile.readline(65537)
         if len(self.raw_requestline) > 65536:
             self.requestline = ''
@@ -224,7 +255,7 @@ class EmbeddedRequestHandler(WSGIRequestHandler):
         handler.run(self.server.get_app())
 
 class LoggingWSGIRequestHandler(EmbeddedRequestHandler):
-    'Disables logging of WSGI messages.'
+    'Disables logging of WSGI messages (conditionally).'
 
     def log_message(self, format, *args):
         from stuphos import getConfig
@@ -283,12 +314,19 @@ class LoggingMiddleware(MiddlewareMixin):
         source = task.taskName if task is not None else 'web-request'
         meta = request.META.get
 
-        content = json.dumps(dict(remoteAddress = meta('REMOTE_ADDR'),
-                                  userAgent = meta('HTTP_USER_AGENT'),
-                                  method = meta('REQUEST_METHOD'),
-                                  path = meta('PATH_INFO'),
-                                  queryString = meta('QUERY_STRING')),
-                             indent = 1)
+        # session_key = request.COOKIES.get(settings.SESSION_COOKIE_NAME)
+
+        content = dict(remoteAddress = meta('REMOTE_ADDR'),
+                       userAgent = meta('HTTP_USER_AGENT'),
+                       method = meta('REQUEST_METHOD'),
+                       path = meta('PATH_INFO'),
+                       queryString = meta('QUERY_STRING'))
+
+        # if session_key:
+        #     # User info.
+        #     content['sessionKey'] = session_key
+
+        content = json.dumps(content, indent = 1)
 
         key = Key() # (request)
         content = json.dumps(dict(content = key(content), user = request.user.id))
@@ -296,6 +334,7 @@ class LoggingMiddleware(MiddlewareMixin):
         request.session['retention_token_secret'] = key.private
         request.session['retention_token_public'] = key.public
 
+        # debugOn()
         # :system-journal:
         log += dict(source = source, type = 'web-request/json',
                     content = content, timestamp = now())
@@ -435,18 +474,10 @@ class DjangoService(Facility, ServerControl, WSGIServer, Instrument):
         logger.setLevel(logging.INFO)
 
         try:
-            ValidactivateOnce()
             (hostname, port, admin_media_path, debug, certfile, software, urlConf) = \
                 self.GetConfig()
 
-            from django.conf import settings
-            # Doesn't affect the construction of the middleware.
-            # Q: What about getting the config before activation, and setting then?
-            # settings.DEBUG = debug
-
-            # Moved out of constructor because handler relies on it being configured sooner.
-            from django.apps import apps
-            apps.populate(settings.INSTALLED_APPS)
+            djangoConfig()
 
             #wsgi_handler = AdminMediaHandler(WSGIHandler(), admin_media_path)
             wsgi_handler = WSGIHandler()
@@ -473,7 +504,7 @@ class DjangoService(Facility, ServerControl, WSGIServer, Instrument):
         # Start Immediately.
         self.server_start()
 
-        # Force load the sitemap. XXX Hack
+        # Force load the sitemap. XXX :skip: Hack
         ##    from django.core.urlresolvers import get_urlconf, get_resolver
         ##    urlconf = get_urlconf()
         ##    resolver = get_resolver(urlconf)
@@ -527,7 +558,10 @@ class DjangoService(Facility, ServerControl, WSGIServer, Instrument):
 
 @contextmanager
 def ReloadEmbeddedUrlConf(urlConf = None):
-    from django.core.urlresolvers import clear_url_caches
+    try: from django.core.urlresolvers import clear_url_caches
+    except ImportError:
+        from django.urls.base import clear_url_caches
+
     clear_url_caches()
 
     # Hack in the settings file, because this service doesn't compartmentalize
@@ -540,11 +574,15 @@ def ReloadEmbeddedUrlConf(urlConf = None):
     template_dir = dirname(dirname(__file__))
     template_dir = joinpath(template_dir, 'templates')
 
-    if template_dir not in settings.TEMPLATE_DIRS:
-        settings.TEMPLATE_DIRS += (template_dir,)
+    try:
+        if template_dir not in settings.TEMPLATE_DIRS:
+            settings.TEMPLATE_DIRS += (template_dir,)
+
+    except AttributeError:
+        pass
 
 
-    # XXX obseleted reference:
+    # XXX :skip: obseleted reference:
     # if 'web.application' not in settings.INSTALLED_APPS:
     #     settings.INSTALLED_APPS += ('web.application',)
 
@@ -596,5 +634,170 @@ def GetURLConf(indent = ''):
 def importModule(name):
     return __import__(name, locals(), globals(), [''])
 
+
+def djangoConfig():
+    ValidactivateOnce()
+
+    from django.conf import settings
+    # Doesn't affect the construction of the middleware.
+    # Q: What about getting the config before activation, and setting then?
+    # settings.DEBUG = debug
+
+    # Moved out of constructor because handler relies on it being configured sooner.
+    from django.apps import apps
+    apps.populate(settings.INSTALLED_APPS)
+
+
 # Install Front End.
 DjangoService.manage()
+
+
+# Internal WSGI requests (for internal redirects).
+class InternalServer(BaseServer):
+    @classmethod
+    def doRequest(self, serverAddress, method, path, input, **headers):
+        server = self(serverAddress, self.RequestHandler)
+        buf = BytesIO()
+
+        server.set_request(buf, method, path, input, **headers)
+        server._handle_request_noblock()
+
+        return buf
+
+    @classmethod
+    def doRedirectRequest(self, request, path):
+        qs = getQueryString(request.GET)
+        if qs:
+            path += '?'
+            path += qs
+
+        input = request.raw_post_data
+
+        return self.doRequest(request.META['SERVER_ADDRESS'],
+                              request.method, path, input,
+                              **request.headers)
+
+
+    def set_request(self, wfile, method, path, input, **headers):
+        self.request = [wfile, method, path, input, headers]
+
+    def get_request(self):
+        return self.request
+
+
+    class RequestHandler(WSGIRequestHandler):
+        def setup(self):
+            (wfile, method, path, input, headers) = self.request
+
+            # Reconstruct the HTTP request message.
+            rfile = BytesIO('%s %s HTTP/1.1\r\n' % (method, path))
+
+            for (key, value) in headers.items():
+                rfile.write('%s: %s\r\n' % (key, value))
+
+            rfile.write('\r\n')
+
+            if input:
+                rfile.write(input)
+
+            self.rfile = rfile
+            self.wfile = wfile
+
+
+def getQueryString(o):
+    return '&'.join('%s=%s' % kv for kv in o.items())
+
+
+requestInternal = InternalServer.doRequest
+redirectInternal = InternalServer.doRedirectRequest
+
+
+class httpBufferResponse(baseHTTPResponse):
+    '''
+    We can't use HTTPResponse directly because its constructor
+    expects a socket-like object.
+
+    '''
+
+    def __init__(self, fp, *args, **kwd):
+        # This is a hack to get around the sock-only constructor.
+        baseHTTPResponse.__init__(self, self._bufferWrapper(fp),
+                                  *args, **kwd)
+
+    class _bufferWrapper:
+        def __init__(self, fp):
+            # self.makefile = (lambda *args, **kwd: fp)
+            self.fp = fp
+        def makefile(self, *args, **kwd):
+            return self.fp
+
+
+class requestInternalNative:
+    '''
+    menu:
+        context($trigger)::
+            r = request$internal(request, '/page/gen/site/menu')
+
+            response['status'] = r.status
+            response['content'] = r.content
+
+            '''
+
+    def __new__(self, frame, request, path):
+        from stuphos.structure import RequestAdapter
+        if not isinstance(request, RequestAdapter):
+            raise TypeError(f'argument 1: {type(request).__name__}')
+
+        task = frame.task
+
+        @self._pool.task(task)
+        def request():
+            response = redirectInternal(request._request, path)
+
+            with task.billableComputation():
+                return self._response._build(request, response,
+                                             method = request._request.method)
+
+
+    class _response(writeprotected):
+        @classmethod
+        def _build(self, request, response, method = None):
+            response = httpBufferResponse(response, method = method)
+            return self(request, response.status, response.reason,
+                        response.headers, response.read())
+
+        def __init__(self, request, status, message, headers, content):
+            self._request = request
+            self._status = status
+            self._message = message
+            self._headers = headers
+            self._content = content
+
+        def __repr__(self):
+            return '<response %d %r>' % (self.status, self.message)
+
+
+        @property
+        def request(self):
+            return self._request
+
+        @property
+        def status(self):
+            return self._status
+        code = status
+
+        @property
+        def message(self):
+            return self._message
+        reason = message
+
+        @property
+        def headers(self):
+            from stuphos.kernel import vmNewMapping
+            return vmNewMapping(*self._headers.items())
+
+        @property
+        def content(self):
+            return self._content
+
+_requestInternalNative = requestInternalNative
