@@ -10,12 +10,15 @@ from xmlrpc.server import SimpleXMLRPCRequestHandler as XMLRPCRequestHandler
 from urllib.parse import urlparse
 from errno import EINTR
 from select import error as select_error
+from threading import Thread
 
 import xmlrpc.client
 import sys
+import json
 
 from stuphos import enqueueHeartbeatTask, logException
 from stuphos.etc import nth, reraiseSystemException, getSystemExceptionString
+from stuphos.etc.tools.timing import getCurrentTime as now
 
 ALLOWED_PORT_ORIGINS = [8000, 2180, None]
 
@@ -100,7 +103,7 @@ class Deferrable:
         # Go into deferred mode, so that this request handler finishes
         # its construction phase, but the connection isn't closed yet.
         self.deferred = True
-        raise HostRpc.DeferredMethodSignal
+        raise HostRpc.DeferredMethodSignal(self)
 
 
 class HostRpc(XMLRPC):
@@ -109,12 +112,70 @@ class HostRpc(XMLRPC):
 
     class DeferredMethodSignal(Exception):
         # Signals a method that will be finished by another thread!
-        pass
+        def __init__(self, request):
+            self.request = request
 
     class RequestHandler(Deferrable, XMLRPCRequestHandler):
-        # def log_message(self, *args, **kwd):
-        #     # Todo: log to sys/mudlog.
-        #     return XMLRPCRequestHandler.log_message(self, *args, **kwd)
+        def setup(self):
+            # XXX :network:
+            self.request.settimeout(30)
+            XMLRPCRequestHandler.setup(self)
+
+        # def log_message(self, format, request_line, status = '-', *args, **kwd):
+            # Todo: log to sys/mudlog.
+            # return XMLRPCRequestHandler.log_message(self, *args, **kwd)
+            # debugOn()
+
+        def log_request(self, code='-', size='-'):
+            """Log an accepted request.
+
+            This is called by send_response().
+
+            """
+            from http import HTTPStatus
+
+            if isinstance(code, HTTPStatus):
+                code = code.value
+            # self.log_message('"%s" %s %s',
+            #                  self.requestline, str(code), str(size))
+
+            request_line = self.requestline
+            status = code
+            args = ()
+
+            # Reformat logging of RPC.
+
+            # if httpMethod is not 'OPTIONS':
+            rpcMethod = getattr(self, 'method', '-')
+
+            # self.logRpcMessage(rpcMethod, getattr(self, 'params', []))
+
+            if self.ignoreRpcMethod(rpcMethod):
+                return
+
+            httpMethod = request_line.split(' ', 1)[0]
+            args = f'RPC {httpMethod} {rpcMethod} {", ".join(map(str, args))}'
+
+            # debugOn()
+            # Todo: logOperation
+            from sys import stderr
+            stderr.write(f'{self.address_string()} - - [{self.log_date_time_string()}] {repr(args)} {status} -\n')
+
+        @runtime.available(runtime.System.Journal)
+        def logRpcMessage(log, self, method, params):
+            if method == 'session.command' and params and \
+                params[0] in ['do-interpret-script',
+                              'do-interpret-agent']:
+
+                content = json.dumps(dict(code = params[1]))
+
+                log += dict(source = f'rpc:{self.address_string()}',
+                            type = 'webrequest-rpc/command/json',
+                            timestamp = now(),
+                            content = content)
+
+        def ignoreRpcMethod(self, name):
+            return name in ['session.messages', 'session.command']
 
         ##    def setup(self):
         ##        # StreamRequestHandler.setup(self)
@@ -149,10 +210,16 @@ class HostRpc(XMLRPC):
         UNSUPPORTED_METHOD_MSG = re.compile(r'method "' + METHOD_MATCH + r'" is not supported')
 
         # XXX This method is no longer needed since the override is rewritten in the server class:
+        # (However, log message relies on its functionality)
 
         def _dispatch(self, method, params):
             # Intercept method dispatch to see if it was recognized.
             # Also, prepend this handler instance to the arglist.
+
+            # For log_message when this request finishes.
+            self.method = method
+            self.params = params
+
             try: result = self.server._dispatch(self, method, params)
             except HostRpc.DeferredMethodSignal:
                 # Todo: filter out logging for certain methods.
@@ -233,10 +300,13 @@ class HostRpc(XMLRPC):
                 return
 
             self.deferred = False
+
             try:
                 # :set-user: :authentication: note: this only returns a dictionary...
                 self.user = self.authenticate(self.headers.get('Authorization'))
 
+
+                # from python3.9:
                 # Get arguments by reading body of request.
                 # We read this in chunks to avoid straining
                 # socket.read(); around the 10 or 15Mb mark, some platforms
@@ -244,11 +314,52 @@ class HostRpc(XMLRPC):
                 max_chunk_size = 10*1024*1024
                 size_remaining = int(self.headers["content-length"])
                 L = []
+
+                # self.socket.setblocking(0)
+                # self.socket.settimeout(10)
+
                 while size_remaining:
                     chunk_size = min(size_remaining, max_chunk_size)
-                    L.append(self.rfile.read(chunk_size).decode('utf-8'))
+                    # if chunk_size == 2: # XXX :skip: HACK
+                    #     print('setting blocking to 0')
+                    #     self.socket.setblocking(0)
+
+                    chunk = self.rfile.read(chunk_size)
+                    if not chunk:
+                        break
+                    L.append(chunk)
                     size_remaining -= len(L[-1])
-                data = ''.join(L)
+
+                    # print(f'[xmlrpc] {size_remaining}')
+
+                # self.socket.setblocking(1)
+                # self.socket.settimeout(None)
+
+                # if size_remaining:
+                #     print(f'[xmlrpc] incomplete read')
+
+                data = b''.join(L)
+
+                data = self.decode_request_content(data)
+                if data is None:
+                    # print('[xmlrpc] response has been sent')
+                    return #response has been sent
+
+
+                # debugOn()
+
+                # max_chunk_size = 10*1024*1024
+                # size_remaining = int(self.headers["content-length"])
+                # L = []
+                # while size_remaining:
+                #     chunk_size = min(size_remaining, max_chunk_size)
+
+                #     # XXX :skip: if unicode characters are included, the size_remaining will be wrong..?
+                #     # Meaning we'll read more than is available and block (forever).
+                #     L.append(self.rfile.read(chunk_size).decode('utf-8'))
+                #     size_remaining -= len(L[-1])
+                # data = ''.join(L)
+
 
                 # This qualifies as a "preflight" because of POST method and content-type.
                 self.origin = self.headers.get('Origin')
@@ -268,6 +379,8 @@ class HostRpc(XMLRPC):
                             data, getattr(self, '_dispatch', None)
                         )
 
+                # print(f'[xmlrpc] response: {response}')
+
             except HostRpc.DeferredMethodSignal:
                 # Done (for now).
                 ##    print 'tracing...'
@@ -304,6 +417,7 @@ class HostRpc(XMLRPC):
         def error_this_request(self, fault = True):
             # Q: Log error?
             e = getSystemExceptionString()
+            # debugOn()
             self.log_message('SERVER-ERROR:\n%s', e)
 
             if fault:
@@ -360,9 +474,17 @@ class HostRpc(XMLRPC):
             response = (response,)
             response = xmlrpc.client.dumps(response, methodresponse=1,
                                        allow_none=self.allow_none, encoding=self.encoding)
+
         except xmlrpc.client.Fault as fault:
             response = xmlrpc.client.dumps(fault, allow_none=self.allow_none,
                                        encoding=self.encoding)
+
+        except xmlrpc.client.ResponseError as e:
+            print(f'[xmlrpc$dispatch] {e}')
+            print(data)
+
+            raise
+
         except self.DeferredMethodSignal:
             raise
 
@@ -373,6 +495,7 @@ class HostRpc(XMLRPC):
                 encoding=self.encoding, allow_none=self.allow_none,
                 )
 
+        # debugOn()
         return response
 
     contentTypeHandlers = {'application/json': None}
@@ -385,10 +508,13 @@ class HostRpc(XMLRPC):
     def handleMethod(self, request, method, *params):
         return self._dispatch(request, method, params)
 
+    def _handleMethodDjango(self, request, method, *params):
+        return self._dispatch(request, method, params)
+
     def handleMethodDjango(self, request, method, *params):
         'Handle a request coming from Django WSGI application.  May defer.'
 
-        return self._dispatch(self.DjangoRequestWrapper(request), method, params)
+        return self._handleMethodDjango(self.DjangoRequestWrapper(request), method, params)
 
     class DjangoRequestWrapper(Deferrable):
         # Powerful bridge between XMLRPC request handler and django request object.
@@ -455,6 +581,10 @@ class HostRpc(XMLRPC):
         handler object instance as the first parameters.
         """
 
+        # print(f'[xmlrpc.host.dispatch] {method} {", ".join(map(repr, params))}')
+        # if method == 'session.command' and params[1] == 'do-interpret-script':
+        #     debugOn()
+
         func = None
         try:
             # check to see if a matching function has been registered
@@ -477,6 +607,7 @@ class HostRpc(XMLRPC):
 
         if func is not None:
             # Passes this server instance and the request handler object instance.
+            # print(f'[xmlrpc] func: {func}')
             return func(self, request, *params)
         else:
             raise Exception('method "%s" is not supported' % method)
@@ -514,7 +645,9 @@ class HostRpc(XMLRPC):
         try:
             request, client_address = self.get_request()
         except socket.error as e:
-            print(e)
+            if not str(e).startswith('[SSL: SSLV3_ALERT_CERTIFICATE_UNKNOWN]'):
+                print(f'[_handle_request_noblock]: {e}')
+
             return
 
         if self.verify_request(request, client_address):
@@ -527,7 +660,7 @@ class HostRpc(XMLRPC):
             self.shutdown_request(request)
 
     def handle_error(self, request, client_address):
-        from game import syslog # XXX ??
+        from stuphos.system.api import syslog
         from stuphos import logException
         syslog('Exception happened during processing of request from %s' % (client_address,))
         logException(traceback = True)
@@ -540,7 +673,7 @@ class HostRpc(XMLRPC):
     def _get_heartbeat_dispatcher(self, function, result):
         def heartbeat_dispatcher(*args, **kwd):
             enqueueHeartbeatTask(function, *args, **kwd)
-            # XXX This is wrong, isn't it?  It should be called from the heartbeat?
+            # XXX :skip: This is wrong, isn't it?  It should be called from the heartbeat?
             # Or, maybe what I want is for this function to block, waiting for the
             # heartbeat.  In which case, we'll need to program some synchronicity
             # primitives here (like Queue).
@@ -563,7 +696,8 @@ class HostRpc(XMLRPC):
     def start(self):
         if not self.is_running():
             self.__running = True
-            nth(self.serve_cooperatively)
+            Thread(target = self.serve_cooperatively, daemon = True).start()
+            # nth(self.serve_cooperatively)
 
     def stop(self):
         try: del self.__running
@@ -581,5 +715,32 @@ class HostRpc(XMLRPC):
             # This might not be proper form, but it's a shortcut to cleanup.
             self.stop()
 
+
+    # def server_bind(self):
+    #     self.socket.setblocking(0) # XXX :skip: What I want is nonblocking peers.
+    #     return XMLRPC.server_bind(self)
+
+
     ##    def __registry_delete__(self):
     ##        self.stop()
+
+
+def installMarshaller():
+    from stuphos.kernel import StringValue, baseStringClass
+
+    class AgentMarshaller(xmlrpc.client.Marshaller):
+        dispatch = xmlrpc.client.Marshaller.dispatch.copy()
+        dispatch[StringValue] = dispatch[str]
+        dispatch[baseStringClass] = dispatch[str]
+
+        @property
+        def allow_none(self):
+            return True
+
+        @allow_none.setter
+        def allow_none(self, value):
+            'Compat with initializer.'
+
+
+    # This is the interface used by dumps.
+    xmlrpc.client.FastMarshaller = AgentMarshaller
